import express from "express";
import PermissionHelper from "../../helpers/permissionHelper";
import preventWebapiAccess from "../../middleware/preventWebApiAccess";

import award from "./configuration/award";
import communicationType from "./configuration/communicationType";
import executivePosition from "./configuration/executivePosition";
import membershipStatus from "./configuration/membershipStatus";
import qualification from "./configuration/qualification";
import salutation from "./configuration/salutation";
import calendarType from "./configuration/calendarType";
import queryStore from "./configuration/queryStore";
import template from "./configuration/template";
import templateUsage from "./configuration/templateUsage";
import newsletterConfig from "./configuration/newsletterConfig";

import member from "./club/member";
import protocol from "./club/protocol";
import calendar from "./club/calendar";
import queryBuilder from "./club/queryBuilder";
import newsletter from "./club/newsletter";
import listprint from "./club/listprint";

import role from "./management/role";
import user from "./management/user";
import invite from "./management/invite";
import api from "./management/webapi";
import backup from "./management/backup";

var router = express.Router({ mergeParams: true });

router.use(
  "/award",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "configuration", module: "award" },
    { requiredPermissions: "read", section: "club", module: "member" },
  ]),
  award
);
router.use(
  "/communicationtype",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "configuration", module: "communication_type" },
    { requiredPermissions: "read", section: "club", module: "member" },
  ]),
  communicationType
);
router.use(
  "/executiveposition",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "configuration", module: "executive_position" },
    { requiredPermissions: "read", section: "club", module: "member" },
  ]),
  executivePosition
);
router.use(
  "/membershipstatus",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "configuration", module: "membership_status" },
    { requiredPermissions: "read", section: "club", module: "member" },
  ]),
  membershipStatus
);
router.use(
  "/qualification",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "configuration", module: "qualification" },
    { requiredPermissions: "read", section: "club", module: "member" },
  ]),
  qualification
);
router.use(
  "/salutation",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "configuration", module: "salutation" },
    { requiredPermissions: "read", section: "club", module: "member" },
  ]),
  salutation
);
router.use(
  "/calendartype",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "configuration", module: "calendar_type" },
    { requiredPermissions: "read", section: "club", module: "calendar" },
  ]),
  calendarType
);
router.use(
  "/querystore",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "configuration", module: "query_store" },
    { requiredPermissions: "read", section: "club", module: "listprint" },
  ]),
  queryStore
);
router.use("/template", PermissionHelper.passCheckMiddleware("read", "configuration", "template"), template);
router.use(
  "/templateusage",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "configuration", module: "template_usage" },
    { requiredPermissions: "read", section: "configuration", module: "template" },
  ]),
  templateUsage
);
router.use(
  "/newsletterconfig",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "configuration", module: "newsletter_config" },
    { requiredPermissions: "read", section: "configuration", module: "communication_type" },
  ]),
  newsletterConfig
);

router.use("/member", PermissionHelper.passCheckMiddleware("read", "club", "member"), member);
router.use(
  "/protocol",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "club", module: "protocol" },
    { requiredPermissions: "read", section: "club", module: "member" },
  ]),
  protocol
);
router.use("/calendar", PermissionHelper.passCheckMiddleware("read", "club", "calendar"), calendar);
router.use(
  "/querybuilder",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "club", module: "query" },
    { requiredPermissions: "read", section: "configuration", module: "query_store" },
  ]),
  queryBuilder
);
router.use(
  "/newsletter",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "club", module: "newsletter" },
    { requiredPermissions: "read", section: "club", module: "member" },
    { requiredPermissions: "read", section: "club", module: "calendar" },
    { requiredPermissions: "read", section: "club", module: "query" },
    { requiredPermissions: "read", section: "configuration", module: "query_store" },
  ]),
  newsletter
);
router.use("/listprint", PermissionHelper.passCheckMiddleware("read", "club", "listprint"), listprint);

router.use("/role", PermissionHelper.passCheckMiddleware("read", "management", "role"), role);
router.use(
  "/user",
  PermissionHelper.passCheckSomeMiddleware([
    { requiredPermissions: "read", section: "management", module: "user" },
    { requiredPermissions: "read", section: "management", module: "role" },
  ]),
  user
);
router.use("/invite", PermissionHelper.passCheckMiddleware("read", "management", "user"), invite);
router.use("/webapi", preventWebapiAccess, PermissionHelper.passCheckMiddleware("read", "management", "webapi"), api);
router.use(
  "/backup",
  preventWebapiAccess,
  PermissionHelper.passCheckMiddleware("read", "management", "backup"),
  backup
);

export default router;
